;
.include "Header.inc"       ;Label und Werte Zuweisungen
.include "Hardware.inc"     ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000..

 rjmp  _reset               ;..mit der RESET-Vectortabelle

.include "IRQm88PA.inc"     ;Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren
;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,High(RAMEND)        ;RAMEND, also das Ende vom SRAM, ist in der.. 
 out  SPH,a                 ;..Definitions-datei festgelegt  
 ldi  a,Low (RAMEND)
 out  SPL,a
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
 clr  flag_reg              ;Allgemeines Flaggenregister
 clr  ovf0_zaehler          ;Timer0 Overflowzaehler
 clr  key_reg               ;Tasterentprellregister
 clr  null                  ;Register mit dem Inhalt Null
;
;Ports Erstkonfiguration
; 
 sbi  PLA_PORT,led.pla      ;PraogrammLaufAnzeige-LED aus ( Vorerst Pullup )
 sbi  PLA_DDR, led.pla      ;PORT-Bit als Ausgang f. PLA-LED
;
; USART Baudrate einstellen
;
 ldi  a,#ubrr               ;f. UBRRH bei 9600 Baud mit 1 MHz int. RC-Oszilator
 sts  UBRR0H,null           ;Baudrate 9600 Baud im Auslieferungszustand d.h...
 sts  UBRR0L,a              ;..mit internem 1 MHz RC-Oszillator
;
;USART Format ( 8,N,1 ) ist bereits nach dem Reset voreingestellt, aber um die
;9600 Baud zu erreichen muss U2X0 ( Double the USART Transmission Speed ) ein-
;geschaltet werden. 
;
 ldi  a,1<<U2X0             ;Double the USART Transmission Speed..
 sts  UCSR0A,a              ;..einstellen, um 9600 Baud zu erzielen
;
;USART Sendeschieberegisterinterrupt und Sender ( Transmitter ) freigeben
;
 ldi  a,1<<TXEN0  ;USART Sendeschieberegisterinterrupt freigeben...1<<TXCIE0|
 sts  UCSR0B,a              ;...und USART-Sendeeinheit einschalten
;
;Timer/Counter0: Overflow Interrupt initialisieren
;
 /*ldi  a,1<<TOIE0            ;Timer0 Overflow Interrupt freigeben...
 sts  TIMSK0,a              ;...
 ldi  a,1<<CS02             ;Timer0 Vorteiler => 256...
 out  TCCR0B,a              ;...
*/;
;Timer/Counter0: Fast PWM-Mode ( 7 ) initialisieren, NON-INVERT, ohne Vorteiler.
;Clear OC0B on Compare Match, set OC0B at Bottom. OC0B ist der PWM-Pinn,
;da OCR0A ja den TOP-Wert enthlt. OC0A = toggeln
;
 ldi  a,#tc0_max            ;Zhlumfang erstmal Maximum...
 out  Z_UMFANG,a            ;...einstellen...
 
 ldi  a,TOGG0A|INV0B|1<<WGM01|1<<WGM00;Fast PWM-Mode 7..
 out  TCCR0A,a              ;..Clear OC0A on Compare Match, set OC0A at TOP..
 ldi  a,1<<CS00|1<<WGM02    ;...ohne Vorteiler..
 out  TCCR0B,a              ;..Timer/Counter0 starten und..
 sbi  PWM0_DDR,OC0A         ;..PWM-Portpinn als Ausgang  freigeben..
 sbi  PWM0_DDR,OC0B         ;..PWM-Portpinn als Ausgang  freigeben
;
;ADC ( Analog-Digital-Wandler initialisieren ( Free Running Mode )
;125kHz, Interruptbasierend, interne Referenz Avcc am AREF-Pinn mit 100nF
;Digitaleingangspuffer der jeweiligen ADC-Kanaele abschalten
;
 ldi  a,1<<ADEN|1<<ADSC|1<<ADATE|1<<ADIE|1<<ADPS1|1<<ADPS0;IRQ,Systemtakt/8..
 sts  ADCSRA,a              ;..( 125kHz bei 1 MHz ), ADC freigeben & starten
 ldi  a,1<<ADLAR|1<<REFS0   ;10-Bit Ergebnis linksbuendig in ADCH..
 sts  ADMUX,a               ;..Interne. Referenz Avcc
 ldi  a,1<<MUX1|1<<MUX0     ;ADC-Kanaele 0 & 1 auswaehlen und den Digital-..
 sts  DIDR0,a               ;..eingangspuffer abschalten

 sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
;Zaehlumfang ( POTI_Z ) & Tastgrad ( POTI_T ) per RS232 senden
;
_main:
 cli                        ;TESTZWECKE ########################################
 rcall _warte_udre          ;
 ldi  a,'Z'                 ;
 sts  UDR0,a                ;
 rcall _warte_udre          ;
 in   a,Z_UMFANG            ;
 sts  UDR0,a                ;
                            ;
 rcall _warte_udre          ;
 ldi  a,'T'                 ;
 sts  UDR0,a                ;
 rcall _warte_udre          ;
 in   a,TASTGRAD            ;
 sts  UDR0,a                ;
                            ; 
 sei                        ;TESTZWECKE ########################################
;
;ProgrammLaufAnzeige (PLA) in Gang halten
;
_idle: 
 sbr  flag_reg,1<<pla.flg   ;Programmlaufanzeige-Flag quitieren
 
 rjmp _main                 ;Sprung zum Anfang des Hauptprogramms ( Mainloop )
; 
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;U Beginn der verwendeten Unterprogramme
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;
.include "UP.inc";Unterprogramme
;
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;Q Beginn der verwendeten Interrupt Service Routinen ( ISR )
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;
.include "ISR.inc";Interrupt Service Routinen
;
 
.EXIT                       ;Ende des Quelltextes
