;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
; Headerdatei zu PCPWM_M5
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;Der ATmega88PA laeuft mit einem 1,8432 MHz Quarz und wird an 3,3V betrieben.
;
;FUSES Extendet $F9, High $97, Low $EA
;debugWIRE Enable, EEPROM beim Loeschen nicht mitloeschen, 
;Sytemtakt nicht durch 8 teilen, Low-Power Quarz-Oszillator.
;
;Es wird ein schwarzer Wii Nunchuk ausgelesen und seine Werte ueber die USART
;( RS232 spaeter mittels MAX3232 jetzt MAX232 ) an ein Terminalprogramm ge-
;sendet ( Hterm ). Bei meinem TTL <-> RS232-Wandler ist es noetig dort 
;DTR & RTS einzuschalten, da ueber eine "Trickschaltung" der MAX232 hierrueber
;mit Spannung versorgt wird, welche vorher zu messen ist, bevor der MAX daran
;angeschlossen wird.
;
;Ueber den Motortreiber L298N wird der DC-Motor mittels Nunchuk links/rechts
;neigen (X-Achse Beschleunigungssensor) angesteuert.
;
;Ein einmaliger Tastendruck an PD7 ist noetig, um das Ganze zu starten.
;
;Hterm 6 Positionsbytes:
;1. Spalte = Joystick X-Achse
;2. Spalte = Joystick Y-Achse
;3. Spalte = X-Achse Beschleunigungssensor
;4. Spalte = Y-Achse Beschleunigungssensor
;5. Spalte = Z-Achse Beschleunigungssensor
;6. Spalte = Der Rest wie C-Button, Z-Button usw.
;
;Das Auslesen der 16 Kalibrationsbytes ist auskommentiert, sowie das senden der 
;6 Positionsbytes per RS232. Anstelle dessen wird der korigierte PWM-Wert ge-
;sendet und nach CR+LF der Skalierte, welcher in OCR0B geschrieben wird. In 
;Hterm bei " New Line at " CR+LF einstellen und alle 52 Spalten nutzen.
;
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
; Einbinden der Controllerspezifischen Definitionsdatei
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
;
.NOLIST                 ;List-Output ausschalten
.INCLUDE "m88PAdef.inc" ;AVR-Definitionsdatei einbinden
.LIST                   ;List-Output wieder einschalten
;
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
; Arbeitsregister umbenennen
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;
;.DEF   = r0   ;
;.DEF   = r1   ;
;.DEF   = r2   ;
.DEF x_bsl             = r3   ;X-Achse Beschleunigungssensor Low -Byte
.DEF x_bsh             = r4   ;X-Achse Beschleunigungssensor High-Byte
.DEF sfl               = r5   ;Skalierfaktor Low
.DEF sfh               = r6   ;Skalierfaktor High
.DEF twi_data          = r7   ;TWI-Datenbyte 
.DEF null              = r8   ;Register mit dem Inhalt Null
;.DEF   = r9   ;
;.DEF   = r10	;
.DEF ovf2_zaehler      = r11  ;Timer2 Overflow-Zaehler
;.DEF twi_status_check_z= r12    ;TWI-Status-Check Zeiger
.DEF twi_status_check  = r13  ;Vergleichsregister f. den Status-Check
.DEF zaehler           = r14  ;Allgemeiner Zaehler fuer verschiedene Sachen 
.DEF s_sreg            = r15  ;Zum Sichern des Statusregisters ( Nur 1 Takt )

.DEF ia                = r16  ;Interrupt Arbeitsregister ia ( Nur in ISR genutzt )
.DEF ib                = r17  ;Interrupt Arbeitsregister ib ( Nur in ISR genutzt )
;.DEF   = r18  ;     
.DEF flag_reg          = r19  ;Allgemeines Flaggenregister;
.DEF twi_ablauf_nr     = r20  ;Zeiger auf die jeweilige TWI-Aktion
.DEF key_reg           = r21  ;Fuer die Tasterentprellung
.DEF twi_txd           = r22  ;TWI-Datensendezeiger
.DEF twi_rxd           = r23  ;TWI-Datenempfangszeiger  
.DEF a                 = r24  ;Register 24 und 25 kann als universelles..
.DEF b                 = r25  ;..Doppelregister dienen
;.DEF xl               = r26    ;Der X-Zeiger ist die TWI-Statusnummertabellen-.. 
;.DEF xh               = r27    ;..adresse ( twi_status_nr_tab: )
;.DEF yl= r28   ;
;.DEF yh= r29   ;
;.DEF zl             = r30   ;Verwendung als Z-Zeiger, also wie bereits in der..
;.DEF zh             = r31   ;..Definitionsdatei von ATMEL angegeben
;
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
; Konstanten als Symbole
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;
.EQU #ubrr			    = 2		;Wert f. USART Baudrate 38400 Baud
;
;Timer/Counter2
;
.EQU #ovf2_zaehler  = 2		;Bit fuer Timer2 Overflowzaehler ( ca. 1Hz f. PLA )

.EQU #tc0_max       = $FF   ;Maximaler Zhlumfang ( Auflsung )
.EQU #pcpwm_freq    = 87    ;Zhlumfang*2 = Auflsung ) f. 10,6kHz

.EQU NINV0A         = 1<<COM0A1|0<<COM0A0;OC0A (Non-Inverting Mode)
.EQU INV0A          = 1<<COM0A1|1<<COM0A0;OC0A (Interting Mode)
.EQU TOGG0A         = 0<<COM0A1|1<<COM0A0;OC0A (Toggle Mode nur OC0A moeglich)

.EQU NINV0B         = 1<<COM0B1|0<<COM0B0;OC0B (Non-Inverting Mode)
.EQU INV0B          = 1<<COM0B1|1<<COM0B0;OC0B (Interting Mode)

.EQU Z_UMFANG       = OCR0A ;Zhlumfang
.EQU TASTGRAD       = OCR0B ;Tastgrad
;
;ADC
;
.EQU POTI_Z         = ~(1<<MUX3|1<<MUX2|1<<MUX1|1<<MUX0);Poti Zhlumfang ADC 0
.EQU POTI_T         = MUX1  ;Poti Tastgrad auf ADC-Kanal 1
;.EQU #ovf0_zaehler	    = 4		;Wert f. Timer0 Overflowzaehler
;
;Nunchuk Konstanten
; 
.EQU #x_bs_nst     = $7F    ;X-Achse Beschleunigunssensor Neutralstellung
.EQU #skalier      = 384    ;X-Achse Beschleunigunssensor * 1,5 ( 384/256 )
;
;TWI Konstanten
; 
.EQU #twi_sla_w         = $A4   ;Adresse des SLAVE ( Write )
.EQU #twi_sla_r         = $A5   ;Adresse des SLAVE ( Read  )

.EQU #twbr			    =  2	;TWI SCL-Frequenz(ca.102 kHz TWBR=1.2 ca.92kHz)
.EQU #twi_status_anz    = 25    ;Anzahl der Eintraege in der TWI-Statustabelle
.EQU #twi_ablauf_nr_anz = 20;31;20    ;Anzahl der einzelnen TWI-Aktionen
.EQU #twi_rxd_anz       =  6+16    ;Anzahl der einzulesenden Wii Nunchuk Daten
.EQU #twi_pos_anz       =  6    ;Anzahl der Positionsbytes
.EQU #twi_txd_anz       =  5;+2   ;Anzahl der sendenden Wii Nunchuk Daten

.EQU #twi_txd_end       = 17;9;8    ;Senden der Daten beenden
.EQU #twi_data_end      = 17    ;TWI-Ablaufnummer ab der Daten lesen endet.

.EQU #cmd_twi_start		= 1<<TWINT|1<<TWSTA|1<<TWEN|1<<TWIE ;TWI-START  Komando
.EQU #cmd_twi_stop      = 1<<TWINT|1<<TWSTO|1<<TWEN|1<<TWIE ;TWI-STOP   Komando
.EQU #cmd_twi_ack       = 1<<TWINT|1<<TWEA |1<<TWEN|1<<TWIE ;TWI-ACKNOWLEDGE 
.EQU #cmd_twi_action    = 1<<TWINT         |1<<TWEN|1<<TWIE ;TWI-Aktion 
;
;Bitbezeichner fuer das allgemeine Flaggenregister flag_reg
;
;.equ			= 0	;
;.equ			= 1	;
;.equ			= 2	;
.equ  	ska.ftg     = 3 ;Skalieren fertig
.equ	pla.flg		= 4	;Programmlaufanzeige-Flagge
.equ	twi.err 	= 5	;TWI-ERROR-Flagge
;.equ		    = 6	;
;.equ		    = 7	; 
;
;Bitbezeichner fuer das Tasterentprellregister key_reg
;
;.equ			= 0	;Taster..
;.equ			= 1	;..Prellzaehler
.equ  key.unactuated	= 2	;Taster entprellt unbetaetigt
.equ  key.actuated		= 3	;Taster entprellt betaetigt
;.equ			= 4	;
;.equ	        = 5	;
;.equ	        = 6	; 
;.equ		    = 7	; 